@@ -29,7 +29,7 @@ use swc_common::{
     FileName, Mark, SourceMap, SyntaxContext,
 };
 use swc_config::{
-    config_types::{BoolConfig, BoolOr, BoolOrDataConfig},
+    config_types::{BoolConfig, BoolOr, BoolOrDataConfig, MergingOption},
     merge::Merge,
 };
 use swc_ecma_ast::{EsVersion, Expr, Program};
@@ -57,7 +57,8 @@ use swc_ecma_transforms::{
     optimization::{const_modules, json_parse, simplifier},
     pass::{noop, Optional},
     proposals::{decorators, export_default_from, import_assertions},
-    react, resolver,
+    react::{self, default_pragma, default_pragma_frag},
+    resolver,
     typescript::{self, TSEnumConfig},
     Assumptions,
 };
@@ -323,15 +324,16 @@ impl Options {
 
         let mut program = parse(syntax, es_version, is_module)?;
 
-        let mut transform = transform.unwrap_or_default();
+        let mut transform = transform.into_inner().unwrap_or_default();
 
         // Do a resolver pass before everything.
         //
         // We do this before creating custom passes, so custom passses can use the
         // variable management system based on the syntax contexts.
         if syntax.typescript() {
-            assumptions.set_class_methods = !transform.use_define_for_class_fields;
-            assumptions.set_public_class_fields = !transform.use_define_for_class_fields;
+            assumptions.set_class_methods = !transform.use_define_for_class_fields.into_bool();
+            assumptions.set_public_class_fields =
+                !transform.use_define_for_class_fields.into_bool();
         }
 
         program.visit_mut_with(&mut resolver(
@@ -401,7 +403,7 @@ impl Options {
         };
 
         if syntax.typescript() {
-            transform.legacy_decorator = true;
+            transform.legacy_decorator = true.into();
         }
         let optimizer = transform.optimizer;
 
@@ -553,8 +555,8 @@ impl Options {
             // Decorators may use type information
             Optional::new(
                 decorators(decorators::Config {
-                    legacy: transform.legacy_decorator,
-                    emit_metadata: transform.decorator_metadata,
+                    legacy: transform.legacy_decorator.into_bool(),
+                    emit_metadata: transform.decorator_metadata.into_bool(),
                     use_define_for_class_fields: !assumptions.set_public_class_fields
                 }),
                 syntax.decorators()
@@ -566,10 +568,24 @@ impl Options {
                 typescript::strip_with_jsx(
                     cm.clone(),
                     typescript::Config {
-                        pragma: Some(transform.react.pragma.clone()),
-                        pragma_frag: Some(transform.react.pragma_frag.clone()),
+                        pragma: Some(
+                            transform
+                                .react
+                                .pragma
+                                .clone()
+                                .unwrap_or_else(default_pragma)
+                        ),
+                        pragma_frag: Some(
+                            transform
+                                .react
+                                .pragma_frag
+                                .clone()
+                                .unwrap_or_else(default_pragma_frag)
+                        ),
                         ts_enum_config: TSEnumConfig {
-                            treat_const_enum_as_enum: transform.treat_const_enum_as_enum,
+                            treat_const_enum_as_enum: transform
+                                .treat_const_enum_as_enum
+                                .into_bool(),
                             ts_enum_is_readonly: assumptions.ts_enum_is_readonly,
                         },
                         use_define_for_class_fields: !assumptions.set_public_class_fields,
@@ -588,7 +604,7 @@ impl Options {
                 syntax.jsx()
             ),
             pass,
-            Optional::new(jest::jest(), transform.hidden.jest),
+            Optional::new(jest::jest(), transform.hidden.jest.into_bool()),
             Optional::new(
                 dropped_comments_preserver(comments.cloned()),
                 preserve_all_comments
@@ -1070,7 +1086,7 @@ pub struct JscConfig {
     pub syntax: Option<Syntax>,
 
     #[serde(default)]
-    pub transform: Option<TransformConfig>,
+    pub transform: MergingOption<TransformConfig>,
 
     #[serde(default)]
     pub external_helpers: BoolConfig<false>,
@@ -1245,7 +1261,7 @@ impl ModuleConfig {
     }
 }
 
-#[derive(Debug, Default, Clone, Serialize, Deserialize)]
+#[derive(Debug, Default, Clone, Serialize, Deserialize, Merge)]
 #[serde(deny_unknown_fields, rename_all = "camelCase")]
 pub struct TransformConfig {
     #[serde(default)]
@@ -1258,10 +1274,10 @@ pub struct TransformConfig {
     pub optimizer: Option<OptimizerConfig>,
 
     #[serde(default)]
-    pub legacy_decorator: bool,
+    pub legacy_decorator: BoolConfig<false>,
 
     #[serde(default)]
-    pub decorator_metadata: bool,
+    pub decorator_metadata: BoolConfig<false>,
 
     #[serde(default)]
     pub hidden: HiddenTransformConfig,
@@ -1270,17 +1286,17 @@ pub struct TransformConfig {
     pub regenerator: regenerator::Config,
 
     #[serde(default)]
-    pub treat_const_enum_as_enum: bool,
+    pub treat_const_enum_as_enum: BoolConfig<false>,
 
     #[serde(default)]
-    pub use_define_for_class_fields: bool,
+    pub use_define_for_class_fields: BoolConfig<false>,
 }
 
-#[derive(Debug, Default, Clone, Serialize, Deserialize)]
+#[derive(Debug, Default, Clone, Serialize, Deserialize, Merge)]
 #[serde(deny_unknown_fields, rename_all = "camelCase")]
 pub struct HiddenTransformConfig {
     #[serde(default)]
-    pub jest: bool,
+    pub jest: BoolConfig<false>,
 }
 
 #[derive(Debug, Default, Clone, Serialize, Deserialize, Merge)]
